/* Copyright (c) 2017 Felix Kutzner (github.com/fkutzner)

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.

 Except as contained in this notice, the name(s) of the above copyright holders
 shall not be used in advertising or otherwise to promote the sale, use or
 other dealings in this Software without prior written authorization.

*/

#pragma once

#include <istream>

namespace jamsat {
/**
 * \ingroup JamSAT_TestInfrastructure
 *
 * \brief The JamSAT fuzzing entry point. Fuzzing targets must implement this
 * function.
 *
 * During fuzz testing, the fuzzing data (e.g. generated by AFL-Fuzz) is passed
 * to this function by the fuzzing adapter. See
 * http://llvm.org/docs/LibFuzzer.html#fuzz-target for fuzz target
 * implementation hints. To implement a fuzz target, implement this function
 * and link agains a fuzzing adapter (e.g. the generic fuzzing adapter, which
 * results in a binary that can be called e.g. by afl-fuzz).
 *
 * \param[in,out] fuzzerInput    The fuzzing data input stream.
 */
void JamSATFuzzingEntryPoint(std::istream& fuzzerInput);
}
